#include<iostream>
#include<cmath>
#include<vector>
#include<deque>
#include<tuple>
#include<algorithm>
using namespace std;

const int MAXN=2e5, INF=1e9;
const int MAXM=(1<<(int)ceil(log2(MAXN)));

int seg[2*MAXM+1], l[2*MAXM+1], r[2*MAXM+1], nSize[2*MAXM+1];
int a[MAXN], ans[MAXN];

void InitSegTree(int n)
{
    int m=(1<<(int)ceil(log2(n)));
    for(int i=0; i<m; i++)
    {
        seg[i+m]=-INF;
        l[i+m]=-INF;
        r[i+m]=-INF;
        nSize[i+m]=1;
    }

    for(int i=m-1; i>0; i--)
    {
        seg[i]=-INF;
        l[i]=-INF;
        r[i]=-INF;
        nSize[i]=nSize[2*i]+nSize[2*i+1];
    }

    return;
}

void UpdateTree(int idx)
{
    if(idx==0)
    {
        return;
    }

    l[idx]=l[2*idx];
    r[idx]=r[2*idx+1];

    int mid=0;
    if(r[2*idx]>0)
    {
        mid+=r[2*idx];

        if(r[2*idx+1]==nSize[2*idx+1])
        {
            r[idx]+=r[2*idx];
        }
    }

    if(l[2*idx+1]>0)
    {
        mid+=l[2*idx+1];

        if(l[2*idx]==nSize[2*idx])
        {
            l[idx]+=l[2*idx+1];
        }
    }

    seg[idx]=max(mid, max(seg[2*idx], seg[2*idx+1]));

    UpdateTree(idx/2);

    return;
}

void SetNumber(int n, int idx, bool active)
{
    int m=(1<<(int)ceil(log2(n)));
    if(active)
    {
        seg[idx+m]=1;
        l[idx+m]=1;
        r[idx+m]=1;
    }
    else
    {
        seg[idx+m]=-INF;
        l[idx+m]=-INF;
        r[idx+m]=-INF;
    }

    UpdateTree((idx+m)/2);

    return;
}

int main()
{
    ios_base::sync_with_stdio(0);
    cin.tie(0);

    int n, m;
    cin >> n >> m;

    for(int i=0; i<n; i++)
    {
        cin >> a[i];
        a[i]--;
    }

    InitSegTree(n);

    vector<tuple<int, int, int> > queries;
    for(int i=0; i<m; i++)
    {
        int l1, r1;
        cin >> l1 >> r1;
        l1--;

        queries.push_back({l1, r1, i});
    }
    sort(queries.begin(), queries.end());

    int ptrL=0, ptrR=0;
    for(int i=0; i<m; i++)
    {
        int l1, r1, idx;
        tie(l1, r1, idx)=queries[i];

        while(ptrR<r1)
        {
            //cout << "DEBUG ptrR = " << ptrR << "\n";
            SetNumber(n, a[ptrR], true);
            ptrR++;
        }

        if(ptrR>r1)
        {
            while(ptrR>=r1)
            {
                //cout << "DEBUG 2 ptrR = " << ptrR << "\n";
                SetNumber(n, a[ptrR], false);
                ptrR--;
            }
            ptrR++;
        }

        while(ptrL<l1)
        {
            //cout << "DEBUG ptrL = " << ptrL << "\n";
            SetNumber(n, a[ptrL], false);
            ptrL++;
        }

        if(ptrL>l1)
        {
            while(ptrL>=l1)
            {
                //cout << "DEBUG 2 ptrL = " << ptrL << "\n";
                SetNumber(n, a[ptrL], true);
                ptrL--;
            }
            ptrL++;
        }

        ans[idx]=seg[1];

        /*cout << "==========================\n";
        for(int i=1; i<8; i++)
        {
            cout << seg[i] << " ";

            if(i==1 || i==3 || i==7)
            {
                cout << "\n";
            }
        }
        cout << "==========================\n";*/
    }

    for(int i=0; i<m; i++)
    {
        cout << ans[i] << "\n";
    }

    return 0;
}
