#pragma GCC optimize ("O3")

#include <iostream>
#include <cstdint>
#include <queue>
#include <cmath>
#include <algorithm>

#define MOD 1050000011

uint64_t n;
uint64_t m;

uint64_t uintpow(uint64_t base, uint64_t power) {
	uint64_t ret = 1;
	
	while (power) {
		if (power & 1) ret = ret * base;
		base = base * base;
		
		if (ret >= MOD || base >= MOD) {
			return 0;
		}
		
		power >>= 1;
	}
		
	return ret;
}

bool powcmp(uint64_t a, uint64_t b, uint64_t n) {
	// return true if a^n < b^(n+1)
	double ad = a;
	double bd = b;
	double nd = n;
	return ad < bd * pow(bd, 1.0 / nd);
}

uint64_t force(uint64_t n, uint64_t m) {
	uint64_t arr[n];
	for (uint64_t i = 0; i < n; i += 1) arr[i] = 1;
	
	uint64_t ret = 0;
	
	while (true) {
		uint64_t inc = 1;
		
		for (uint64_t i = 1; i < n; i += 1) {
			if (powcmp(arr[i-1], arr[i], i)) {
				inc = 0;
				break;				
			}
		}
		
		if (false && inc) {
			for (uint64_t i = 0; i < n; i += 1) {
				std::cout << arr[i] << " ";
			}
			std::cout << std::endl;
		}
		
		ret += inc;
		
		for (uint64_t i = 0; i < n; i += 1) {
			arr[i] += 1;
			if (i == n - 1 && arr[i] > m) return ret;
			if (arr[i] <= m) break;
			arr[i] = 1;
		}
	}
}

#define CACHE_SIZE 100
uint64_t cache[CACHE_SIZE][CACHE_SIZE][CACHE_SIZE];

uint64_t fast(uint64_t n, uint64_t m_min, uint64_t m_max) {
	if (n < CACHE_SIZE && m_min < CACHE_SIZE && m_max < CACHE_SIZE && cache[n][m_min][m_max] != 0) return cache[n][m_min][m_max];
	
	if (m_min > m_max) return 0;
	if (n == 0) return 1;
	if (n == 1) return m_max - m_min + 1;
	if (n == 2 && m_min == 1) {
		uint64_t ret = 0;
		for (uint64_t i = 1; i <= m_max; i += 1) {
			ret = (uint64_t)(ret + sqrt(i)) % MOD;
		}
		return ret;
	}

	uint64_t ret = 0;
	
	for (uint64_t i = m_min; i <= m_max; i += 1) {
		if (i != 1 && i + n > m_max) break;
		uint64_t p = ceil(pow(i, (double)n/(n-1)));
		uint64_t inc = fast(n-1, p, m_max);
		if (inc == 0) break;
		ret = (ret + inc) % MOD;
	}

	if (n < CACHE_SIZE && m_min < CACHE_SIZE && m_max < CACHE_SIZE) cache[n][m_min][m_max] = ret;
	
	return ret;
}

uint64_t fast2(uint64_t n, uint64_t m_min, uint64_t m_max) {
	if (m_min > m_max) return 0;
	if (n == 0) return 1;
	if (n == 1) return m_max - m_min + 1;
	if (n == 2 && m_min == 1) {
		uint64_t ret = 0;
		for (uint64_t i = 1; i <= m_max; i += 1) {
			ret = (uint64_t)(ret + sqrt(i)) % MOD;
		}
		return ret;
	}

	uint64_t ret = 0;
	
	for (uint64_t i = m_min; i <= m_max; i += 1) {
		if (i != 1 && i + n > m_max) break;
		uint64_t p = ceil(pow(i, (double)n/(n-1)));
		uint64_t inc = fast2(n-1, p, m_max);
		if (inc == 0) break;
		ret = (ret + inc) % MOD;
	}
	
	return ret;
}

void show_all(uint64_t n, uint64_t m) {
	for (uint64_t i = 2; i <= n; i += 1) {
		for (uint64_t j = 2; j <= m; j += 1) {
			std::cout << fast(i, 1, j) << " " << std::flush;
			// std::cout << (fast(i, 1, j) == fast2(i, 1, j)) << " " << std::flush;
		}
		std::cout << std::endl;
	}
}

int main() {
	std::ios_base::sync_with_stdio(0);
	std::cin.tie(0);
	std::cout.tie(0);
	
	std::cin >> n >> m;
	
	// show_all(n, m);
	std::cout << fast(n, 1, m) << std::endl;
}