/*
    Вечеряй, Радо, вечеряй!

    Да ви еба в интерполациите.
*/

#include <algorithm>
#include <iostream>
#include <numeric>
#include <cassert>
#include <vector>
#include <cmath>
#include <queue>
#include <stack>

typedef long long llong;
const int MAXN = 200000 + 10;
const int INF  = 1e9;

int n, q;
struct DSU
{
    struct Roll
    {
        int *pos;
        int val;
        int t;
    };

    std::stack <Roll> rollback;
    int par[MAXN];
    int dep[MAXN];
    int sz[MAXN];
    int time;
    int ans;

    void build()
    {
        while (!rollback.empty()) rollback.pop();
        std::iota(par + 1, par + 1 + n, 1);
        std::fill(dep + 1, dep + 1 + n, 1);
        std::fill(sz + 1, sz + 1 + n, 1);
        time = 0;
        ans = 1;
    }

    int find(int node)
    {
        if (node == par[node]) return node;
        return par[node] = find(par[node]);
    }

    void connect(int x, int y)
    {
        x = find(x);
        y = find(y);

        if (x == y)
        {
            return;
        }

        if (dep[x] > dep[y])
        {
            std::swap(x, y);
        }

        time++;
        if (dep[x] == dep[y])
        {
            rollback.push({&dep[y], dep[y], time});
            dep[y]++;
        }

        rollback.push({&ans, ans, time});
        rollback.push({&par[x], par[x], time});
        rollback.push({&sz[y], sz[y], time});
        par[x] = y;
        sz[y] += sz[x];
        ans = std::max(ans, sz[y]);
    }

    void makeRollback(int to)
    {
        time = to;
        while (!rollback.empty() && rollback.top().t > time)
        {
            *rollback.top().pos = rollback.top().val;
            rollback.pop();
        }
    }
};

int bucketSize;
int inBucket(int pos)
{
    return pos / bucketSize;
}

struct Query
{
    int l, r;
    int idx;

    inline friend bool operator < (const Query &a, const Query &b)
    {
        if (inBucket(a.l) != inBucket(a.r)) return a.l < b.l;
        return a.r < b.r;
    }
};

int a[MAXN];
int ans[MAXN];
bool active[MAXN];
Query query[MAXN];
DSU dsu;

void add(int val)
{
    active[val] = true;
    if (active[val - 1]) dsu.connect(val, val - 1);
    if (active[val + 1]) dsu.connect(val, val + 1);
}

void remove(int val)
{
    active[val] = false;
}

void solve()
{
    int ptr;
    int border;
    int last = -1;
    bucketSize = n + 1; //sqrt(n + q);
    std::sort(query + 1, query + 1 + q);
    for (int i = 1 ; i <= q ; ++i)
    {
        int qL = query[i].l;
        int qR = query[i].r;

        if (inBucket(qL) != last)
        {
            dsu.build();
            last = inBucket(qL);
            ptr = std::min((inBucket(qL) + 1) * bucketSize - 1, n);
            border = (inBucket(qL) + 1) * bucketSize;
        }

        if (inBucket(qL) == inBucket(qR))
        {
            for (int i = qL ; i <= qR ; ++i)
            {
                add(a[i]);
            }

            ans[query[i].idx] = dsu.ans;
            dsu.makeRollback(0);

            for (int i = qL ; i <= qR ; ++i)
            {
                remove(a[i]);
            }
        } else
        {
            while (ptr < qR) add(a[++ptr]);
            int rollbackTime = dsu.time;
            for (int i = qL ; i < border ; ++i)
            {
                add(a[i]);
            }

            ans[query[i].idx] = dsu.ans;
            dsu.makeRollback(rollbackTime);
            for (int i = qL ; i < border ; ++i)
            {
                remove(a[i]);
            }
        }
    }
}

void output()
{
    for (int i = 1 ; i <= q ; ++i)
    {
        std::cout << ans[i] << '\n';
    }
}

void input()
{
    std::cin >> n >> q;
    for (int i = 1 ; i <= n ; ++i)
    {
        std::cin >> a[i];
    }

    for (int i = 1 ; i <= q ; ++i)
    {
        std::cin >> query[i].l >> query[i].r;
        query[i].idx = i;
    }
}

void fastIOI()
{
    std::ios_base :: sync_with_stdio(0);
    std::cout.tie(nullptr);
    std::cin.tie(nullptr);
}

int main()
{
    fastIOI();
    input();
    solve();
    output();

    return 0;
}
